@@ -6,6 +6,7 @@ require("callinfo.common.php");
 function callinfo_clear() {
 	$objResponse = new xajaxResponse();
 	$objResponse->assign("callinfo", "style.display", "none");
+
 	return $objResponse;
 }
 
@@ -33,8 +34,8 @@ function callinfo_check() {
 	$callinfo_refresh = (float)$callinfo_refresh;
 	$js = sprintf("last_update = '%s';", urlencode($_SESSION['last_update']));
 	if($callinfo_refresh)
-		$js .= sprintf("callinfo_refresh = %s; setTimeout('update_check()', %s);",
-				$callinfo_refresh * 1000, $callinfo_refresh);
+		$js .= sprintf("callinfo_refresh = %s; setTimeout('update_check()', callinfo_refresh);",
+				$callinfo_refresh * 1000);
 	$objResponse->script($js);
 	return $objResponse;
 }
@@ -46,6 +47,7 @@ define("STATE_UNREGISTERED", 3);
 define("STATE_REGISTERED", 4);
 define("STATE_INUSE", 5);
 define("STATE_RINGING", 6);
+define("STATE_INUSERINGING", 7);
 
 $states = array(STATE_UNKNOWN => array('state-unknown.png', _("Unknown")),
 		STATE_UNREACHABLE => array('state-unreachable.png', _("Unreachable")),
@@ -53,7 +55,8 @@ $states = array(STATE_UNKNOWN => array('state-unknown.png', _("Unknown")),
 		STATE_UNREGISTERED => array('state-unregistered.png', _("Unregistered")),
 		STATE_REGISTERED => array('state-idle.png', _("Registered")),
 		STATE_INUSE => array('state-inuse.png', _("In use")),
-		STATE_RINGING => array('state-ringing.png', _("Ringing")));
+		STATE_RINGING => array('state-ringing.png', _("Ringing")),
+		STATE_INUSERINGING => array('state-inuse+ringing.png', _("In use & Ringing")));
 	
 $chans = array(1 => array('chan-inuse.png', _("Ringing")),
 		2 => array('chan-ringing.png', _("Talking")));
@@ -126,12 +129,14 @@ function update_blf($objResponse, $old_update) {
 		        #EXTENSION_RINGING     =  8
 		        #EXTENSION_ONHOLD      = 16
 
-			if($row['state'] == -2) {
+			if(($row['state'] == -2) || ($row['state'] & 4)) {
 				$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_UNREACHABLE);
 			} else if($row['state'] == -1) {
 				$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_UNREGISTERED);
 			} else if($row['state'] == 0) {
 				$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_REGISTERED);
+			} else if(($row['state'] & 8) && ($row['state'] & 19)) {
+				$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_INUSERINGING);
 			} else if($row['state'] & 8) {
 				$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_RINGING);
 			} else if($row['state'] & 19) {
@@ -143,13 +148,22 @@ function update_blf($objResponse, $old_update) {
 
 		$_SESSION['blf_states'][$row['ext']]['channels'][$row['channel']] = array('state' => $row['state'],
 									'callerid' => $row['callerid'],
-									'calleridname' => $row['calleridname']);
+									'calleridname' => $row['calleridname'],
+									'peerchannel' => $row['peerchannel']);
+		if($ext['state'] == STATE_INUSERINGING)
+			continue;
 		switch($row['state']) {
 			case 1:	# In use
-				$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_INUSE);
+				if($ext['state'] == STATE_RINGING)
+					$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_INUSERINGING);
+				else
+					$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_INUSE);
 				break;
 			case 2:	# Ringing
-				$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_RINGING);
+				if($ext['state'] == STATE_INUSE)
+					$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_INUSERINGING);
+				else
+					$_SESSION['blf_states'][$row['ext']]['state'] = max($ext['state'], STATE_RINGING);
 				break;
 		}
 	}
@@ -158,6 +172,12 @@ function update_blf($objResponse, $old_update) {
 	# Look for each configured BLF button if it contains an extension that was updated.
 	# If so execute xajax updates
 	#
+	$blf_ringing_prefix = get_config_value(Null, 'blf_ringing_prefix');
+	$allow_pickup       = get_config_value(Null, 'blf_allow_pickup');
+	$allow_steal        = get_config_value(Null, 'blf_allow_steal');
+	$show_ringing       = get_config_value(Null, 'blf_show_ringing');
+	$show_connected     = get_config_value(Null, 'blf_show_connected');
+
 	foreach($_SESSION['blf'] as $blf) {
 		if(!array_intersect($blf['ext'], $updated_exts))	# No updates for BLF button
 			continue;
@@ -166,24 +186,61 @@ function update_blf($objResponse, $old_update) {
 		$state = STATE_UNKNOWN;
 		$channels = array();
 		foreach($blf['ext'] as $ext) {
-			$state = max($_SESSION['blf_states'][$ext]['state'], $state);
-			$channels = array_merge($channels, $_SESSION['blf_states'][$ext]['channels']);
+			if((($_SESSION['blf_states'][$ext]['state'] == STATE_RINGING) && ($state == STATE_INUSE)) ||
+				(($_SESSION['blf_states'][$ext]['state'] == STATE_INUSE) && ($state == STATE_RINGING)))
+				$state = STATE_INUSERINGING;
+			else
+				$state = max($_SESSION['blf_states'][$ext]['state'], $state);
+			if(is_array($_SESSION['blf_states'][$ext]['channels']))
+				$channels = array_merge($channels, $_SESSION['blf_states'][$ext]['channels']);
 		}
 
 		$objResponse->assign("led-$extid", "src", "images/" . $states[$state][0]);
 		$objResponse->assign("led-$extid", "title", $states[$state][1]);
+		if($blf['phone']) {
+			if(($state == STATE_RINGING) || ($state == STATE_INUSERINGING)) {
+				$phone = $blf_ringing_prefix . $blf['phone'];
+				$title = _("Pickup");
+			} else {
+				$phone = $blf['phone'];
+				$title = _("Dial");
+			}
+			$objResponse->addEvent("dial-$extid", "onclick", "xajax_dial(\"$phone\")");
+			$objResponse->assign("dial-$extid", "title", $title);
+		}
 
-		$objResponse->clear("channels-$extid", "innerHTML");
-		foreach($channels as $id => $channel) {
-			#$str = sprintf("%s:%s:%s:%s<br/>", $id, $channel['state'], $channel['callerid'], $channel['calleridname']);
-			$str = sprintf("<img class='channel' src='images/%s' title='%s' id='%s'> ",
-					$chans[$channel['state']][0],
-					$chans[$channel['state']][1],
-					$id);
-			$str .= sprintf("&lt;%s&gt; %s<br/>",
-					htmlspecialchars($channel['callerid']),
-					htmlspecialchars($channel['calleridname']));
-			$objResponse->append("channels-$extid", "innerHTML", $str);
+		if($show_ringing || $show_connected) {
+			$objResponse->clear("channels-$extid", "innerHTML");
+			if($channels)
+				foreach($channels as $id => $channel) {
+					$link = '';
+					if($channel['state'] == 1) {		# Connected
+						if(!$show_connected)
+							continue;
+						if($allow_steal) {
+							$link = sprintf("<a href='#' onclick='xajax_redirect(\"%s\")' title='%s'>",
+									$channel['peerchannel'], _("Steal channel"));
+						}
+					} else if($channel['state'] == 2) {	# Ringing
+						if(!$show_ringing)
+							continue;
+						if($allow_pickup) {
+							$link = sprintf("<a href='#' onclick='xajax_redirect(\"%s\")' title='%s'>",
+									$channel['peerchannel'], _("Pickup channel"));
+						}
+					}
+					$str = sprintf("%s<img class='channel' src='images/%s' title='%s'> ",
+							$link,
+							$chans[$channel['state']][0],
+							$chans[$channel['state']][1]);
+					$str .= sprintf("&lt;%s&gt; %s",
+							htmlspecialchars($channel['callerid']),
+							htmlspecialchars($channel['calleridname']));
+					if($link)
+						$str .= "</a>";
+					$str .= "<br/>";
+					$objResponse->append("channels-$extid", "innerHTML", $str);
+				}
 		}
 	}
 
