@@ -138,3 +138,12 @@ function select_options(id) {
 		options[i].selected = true;
 }
 
+/*
+ * Remove all children of a node
+ */
+function remove_children(id) {
+	children = document.getElementById(id).childNodes;
+	for (i = children.length - 1; i >= 0; i--) {
+		xajax.dom.remove(children.item(i));
+	};
+}
