@@ -25,20 +25,20 @@ class history extends listobj {
 
 		$html = sprintf("<td>%s</td>", $row['stime']);
 		$html .= sprintf("<td>%s</td>", $this->get_extension($row['src']));
-		if($row['src_calleridname'] && $row['src_calleridname'] != '<Unknown>')
+		if($row['src_calleridname'] && strcasecmp($row['src_calleridname'], '<Unknown>'))
 			$html .= sprintf("<td>%s<br/>&lt;%s&gt;", htmlspecialchars($row['src_calleridname']), htmlspecialchars($row['src_callerid']));
 		else
 			$html .= sprintf("<td>%s", htmlspecialchars($row['src_callerid']));
-		if($_SESSION['ext'] && $row['src_callerid'])
+		if($_SESSION['ext'] && $row['src_callerid'] && strcasecmp($row['src_callerid'], '<Unknown>'))
 			$html .= sprintf(" <a href='#' onclick='xajax_dial(\"%s\")'><img class='dial' src='images/dial.png' title='" . _("Dial") . "'></a>",
 					 $row['src_callerid']);
 		$html .= "</td>";
 		$html .= sprintf("<td>%s</td>", $this->get_extension($row['dest']));
-		if($row['dest_calleridname'] && $row['dest_calleridname'] != '<Unknown>')
+		if($row['dest_calleridname'] && strcasecmp($row['dest_calleridname'], '<Unknown>'))
 			$html .= sprintf("<td>%s<br/>&lt;%s&gt;", htmlspecialchars($row['dest_calleridname']), htmlspecialchars($row['dest_callerid']));
 		else
 			$html .= sprintf("<td>%s", htmlspecialchars($row['dest_callerid']));
-		if($_SESSION['ext'] && $row['dest_callerid'])
+		if($_SESSION['ext'] && $row['dest_callerid'] && strcasecmp($row['dest_callerid'], '<Unknown>'))
 			$html .= sprintf(" <a href='#' onclick='xajax_dial(\"%s\")'><img class='dial' src='images/dial.png' title='" . _("Dial") . "'></a>",
 					 $row['dest_callerid']);
 		$html .= "</td>";
@@ -69,7 +69,7 @@ class history extends listobj {
 		$term = sprintf("ILIKE '%%%s%%'", pg_escape_string($string));
 		$cond = "WHERE (src $term OR dest $term OR src_callerid $term OR src_calleridname $term OR dest_callerid $term OR dest_calleridname $term)";
 		if(!$_SESSION['admin'])
-			$cond .= sprintf(" AND dest ILIKE '%s%%'", $_SESSION['ext']);
+			$cond .= sprintf(" AND (dest ILIKE '%s-%%' OR src ILIKE '%s-%%')", $_SESSION['ext'], $_SESSION['ext']);
 		if($count)
 			$query = "SELECT count(*) FROM history $cond";
 		else {
