@@ -141,6 +141,7 @@ function html_userprefSubMenu()
 {
 	$html = "<li><a href='#' onclick='xajax_up_display(\"\");return false;'>" . _("Preferences") . "</a></li>";
 	$html .= "<li><a href='#' onclick='xajax_up_blf_display(\"\");return false;'>" . _("Busy-Lamp-Field") . "</a></li>";
+	$html .= "<li><a href='#' onclick='xajax_up_callinfo_display(\"\");return false;'>" . _("Callinfo") . "</a></li>";
 	$html .= "<li><a href='#' onclick='xajax_up_pw_display(\"\");return false;'>" . _("Change password") . "</a></li>";
 	return $html;
 }
@@ -170,6 +171,18 @@ function up_blf_save($data) {
 	return $ubp->save($data);
 }
 
+require("usercallinfopref.php");
+function up_callinfo_display() {
+	$ucp = new usercallinfopref();
+	$objResponse = $ucp->display();
+	return $objResponse;
+}
+
+function up_callinfo_save($data) {
+	$ucp = new usercallinfopref();
+	return $ucp->save($data);
+}
+
 require("userpwpref.php");
 function up_pw_display() {
 	$upp = new userpwpref();
@@ -185,10 +198,13 @@ function up_pw_save($data) {
 /* Dialing */
 function do_dial($src, $callerid, $dest)
 {
+	# Simplistic check if someone tries to inject AMI commands
+	if(strpos($src . $callerid . $dest, "\n") !== False)
+		return;
+
 	require("ami.php");
 	$ami = new AMI();
 
-	$objResponse = new xajaxResponse();
 	if(!$ami->login())
 		return $ami->error;
 
@@ -210,5 +226,26 @@ function dial($dest)
 	return $objResponse;
 }
 
+function redirect($channel) {
+	# Simplistic check if someone tries to inject AMI commands
+	if(strpos($channel, "\n") !== False)
+		return;
+
+	require("ami.php");
+	$ami = new AMI();
+
+	$objResponse = new xajaxResponse();
+	if(!$ami->login()) {
+		$objResponse->alert($ami->error);
+		return $objResponse;
+	}
+	if(!$ami->redirect(trim($channel), $_SESSION['callerid'])) {
+		$objResponse->alert($ami->error);
+		return $objResponse;
+	}
+
+	return $objResponse;
+}
+
 $xajax->processRequest();
 ?>
