@@ -19,9 +19,9 @@ class userlist extends listobj {
 		return $html;
 	}
 
-	function html_editEntry($entry) {
+	function assign_html_editEntry($objResponse, $id, $entry) {
 		$id = (int)$entry['id'];
-		$html  = "<td colspan='6'><form id='editform$id' name='editform' action='#' onsubmit='xajax_" . $this->prefix . "_saveEntry(xajax.getFormValues(\"editform$id\")); return false'>\n";
+		$html  = "<form id='editform$id' name='editform' action='#' onsubmit='xajax_" . $this->prefix . "_saveEntry(xajax.getFormValues(\"editform$id\")); return false'>\n";
 		$html .= "<input type='hidden' name='id' value='$id'/>";
 		$html .= "<div class='editlabel'>" . _("Username") . ":</div>";
 		$html .= "<input class='editusername' type='text' id='username' name='username' value='" . $entry['username'] . "'/><br/>\n";
@@ -46,15 +46,18 @@ class userlist extends listobj {
 		$html .= "<input type='submit' value='" . _("Save") . "'/>\n";
 		$html .= "<input type='button' value='" . _("Back") . "' onclick='xajax_" . $this->prefix . "_search(document.getElementById(\"searchbox\").value)'/>\n";
 		$html .= "<input type='button' value='" . _("Delete entry") . "' onclick='xajax_" . $this->prefix . "_deleteEntry($id,document.getElementById(\"searchbox\").value)'/>";
-		$html .= "</div></form></td>";
+		$html .= "</div></form>";
 
-		return $html;
+		$objResponse->create("row$id", "td", "td$id");
+		$objResponse->assign("td$id", "colSpan", 6);
+		$objResponse->assign("td$id", "innerHTML", $html);
 	}
 
 	function html_row($row) {
 		$html = sprintf("<td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td>",
 				 $row['username'], ($row['admin'] == 't') ? "X":"", $row['ext'], $row['callerid'], gethostbyname($row['gshostname']), $row['phone']);
-		$html .= "<td><a href='#' onclick='xajax_" . $this->prefix . "_editEntry(" . $row['id'] . "); return false;'>E</a></td>";
+		$html .= "<td><a href='#' onclick='xajax_" . $this->prefix . "_editEntry(" . $row['id'] . "); return false;'>";
+		$html .= "<img class='edit' src='images/edit.png' title='" . _("Edit") . "'></a></td>";
 
 		return $html;
 	}
