@@ -51,7 +51,21 @@ class userpref extends prefobj {
 						$row['id'],
 						($config['general_defaultgid'] == $row['id']) ? ' selected':'',
 						$row['groupname']);
-		$html .= "</select><span class='prefexpl'>" . _("Preselected group for newly created phonbook entries.") . "</span><br/>\n";
+		$html .= "</select><span class='prefexpl'>" . _("Preselected group for newly created phonebook entries.") . "</span><br/>\n";
+
+		if(!get_config_value(Null, 'accountindex_fix')) {
+			$html .= "<div class='editlabel'>" . _("Default account") . ":</div>";
+			$html .= "<select class='editpref' name='general_defaultaccount'>\n";
+			$defaultaccount = $config['general_defaultaccount'];
+			$accountindex_max = get_config_value(Null, 'accountindex_max', 6);
+			for($i = 0; $i < $accountindex_max; $i++) {
+				$html .= sprintf("<option value='%d'%s>" . _("Account %d") . "</option>\n",
+							$i, ($i == $defaultaccount) ? " selected":"", $i + 1);
+			}
+			$html .= "</select>\n";
+			$html .= "<span class='prefexpl'>" . _("Preselected account for newly created phonebook entries.") . "</span><br/>\n";
+		}
+
 		/*
 		$html .= "<div class='editlabel'>" . _("Language") . ":</div>";
 		$html .= "<select class='editpref' name='LC_ALL'>\n";
