@@ -19,6 +19,9 @@ update   = False
 registry = {}
 calls    = {}
 
+#gs_idle_updated = {}
+gs_idle_update  = {}
+
 def get_config_value(key):
     """
     Get value from config option from DB.
@@ -48,6 +51,17 @@ def get_calleridname(callerid, calleridname):
             calleridname += row['info']
     return calleridname
 
+#def peers_idle_update(extension, hangup = False):
+def peers_idle_update(extension):
+    cur.execute("SELECT value FROM config WHERE key LIKE 'callinfo_uri_%%' AND uid in (SELECT id from users WHERE ext = %s)",
+                    (extension,))
+    if cur.rowcount:
+        result = cur.fetchall()
+        for row in result:
+            gs_idle_update[row['value']]  = True
+    #if hangup and gs_idle_updated.has_key(extension):
+    #    gs_idle_update[extension] = True
+
 STATE_UNREACHABLE  = 0
 STATE_REACHABLE    = 1
 STATE_UNREGISTERED = 2
@@ -91,7 +105,10 @@ class States:
         global update
 
         self.get_idstate(chan['channel'])
-        del self.states[self.get_extension(chan['channel'])][chan['channel']]
+        extension = self.get_extension(chan['channel'])
+        del self.states[extension][chan['channel']]
+        #peers_idle_update(extension, hangup = True)
+        peers_idle_update(extension)
 
         cur.execute("SELECT DISTINCT ext FROM states WHERE channel = %s OR peerchannel = %s",
                     (chan['channel'], chan['channel']))
@@ -135,25 +152,34 @@ class States:
                          src['callerid'], src['calleridname']))
         update = True
 
-    def link(self, dest):
+    def link(self, src, dest):
         global update
 
+        src_chan = self.get_idstate(dest['channel'])
+        src_chan['state']  = CHAN_INUSE
         dest_chan = self.get_idstate(dest['channel'])
         dest_chan['state']  = CHAN_INUSE
 
-        """
-        cur.execute("SELECT DISTINCT ext FROM states WHERE peerchannel = %s OR channel = %s", (dest['channel'], dest['channel']))
-        if cur.rowcount:
-            update = True
-            result = cur.fetchall()
-            for row in result:
-                cur.execute("UPDATE states SET ts = now() WHERE ext = %s", (row['ext'],))
-        """
+        cur.execute("UPDATE states SET state = %s, ts = now() WHERE channel = %s AND peerchannel = %s",
+                    (src_chan['state'], src['channel'], dest['channel']))
+        if not cur.rowcount:
+            cur.execute("INSERT INTO states (ext, state, channel, peerchannel, callerid, calleridname, ts)"
+                        " VALUES (%s, %s, %s, %s, %s, %s, now())",
+                        (self.get_extension(src['channel']),
+                         src_chan['state'], src['channel'],
+                         dest['channel'],
+                         dest['callerid'], dest['calleridname']))
 
-        cur.execute("UPDATE states SET state = %s, ts = now() WHERE peerchannel = %s OR channel = %s",
-                    (dest_chan['state'], dest['channel'], dest['channel']))
-        if cur.rowcount:
-            update = True
+        cur.execute("UPDATE states SET state = %s, ts = now() WHERE channel = %s AND peerchannel = %s",
+                    (dest_chan['state'], dest['channel'], src['channel']))
+        if not cur.rowcount:
+            cur.execute("INSERT INTO states (ext, state, channel, peerchannel, callerid, calleridname, ts)"
+                        " VALUES (%s, %s, %s, %s, %s, %s, now())",
+                        (self.get_extension(dest['channel']),
+                         dest_chan['state'], dest['channel'],
+                         src['channel'],
+                         src['callerid'], src['calleridname']))
+        update = True
 
     def set_callerid(self, chan):
         global update
@@ -168,13 +194,15 @@ class States:
         global update
 
         state = self.get_state('%s-' % extension)
+	print state
         if status == 'Registered':
             state['connection'] = STATE_REGISTERED
         elif status == 'Unregistered':
             state['connection'] = STATE_UNREGISTERED
         elif status == 'Reachable':
             state['connection'] = STATE_REACHABLE
-        elif status == 'Unreachable':
+        else:
+            # 'Unreachable', 'Lagged', 'Request Sent'
             state['connection'] = STATE_UNREACHABLE
 
         cur.execute("UPDATE states SET state = %s, ts = now() WHERE ext = %s AND channel = 'connection'",
@@ -223,7 +251,8 @@ class Channels:
                                                   'calleridname': None})
 
     def create(self, ev):
-        chan = self.get_channel(ev['Channel'])
+        channel = ev['Channel']
+        chan = self.get_channel(channel)
         chan['id']           = ev['Uniqueid']
         chan['callerid']     = ev.get('CallerID', '')
         if not chan['callerid']:
@@ -231,10 +260,14 @@ class Channels:
         chan['calleridname'] = get_calleridname(chan['callerid'], ev['CallerIDName'])
         if ev['State'] == 'Ringing':
             chan['state'] = CHAN_RINGING
+            extension = channel[:channel.rfind('-')]
+            peers_idle_update(extension)
         elif ev['State'] == 'Ring':
             chan['state'] = CHAN_INUSE
         elif ev['State'] == 'Up':
             chan['state'] = CHAN_INUSE
+            extension = channel[:channel.rfind('-')]
+            peers_idle_update(extension)
         else:
             chan['state'] = CHAN_IDLE
         states.create(chan)
@@ -263,11 +296,11 @@ class Channels:
         states.dial(src_chan, dest_chan)
 
     def link(self, src, dest):
-        #chan = self.get_channel(src)
-        #chan['state']    = CHAN_INUSE
-        chan = self.get_channel(dest)
-        chan['state']    = CHAN_INUSE
-        states.link(chan)
+        src  = self.get_channel(src)
+        src['state']    = CHAN_INUSE
+        dest = self.get_channel(dest)
+        dest['state']    = CHAN_INUSE
+        states.link(src, dest)
         
 
 CALL_RINGING = 0
@@ -280,18 +313,18 @@ class Calls:
     def __repr__(self):
         return self.calls.__repr__()
 
-    def get_call(self, destid):
-        return self.calls.setdefault(destid, {})
+    def get_call(self, dest):
+        return self.calls.setdefault(dest, {})
             
     def create(self, ev):
         global update
 
         src    = ev['Source']
-        srcid  = ev['SrcUniqueID']
+        #srcid  = ev['SrcUniqueID']
         dest   = ev['Destination']
-        destid = ev['DestUniqueID']
+        #destid = ev['DestUniqueID']
 
-        call = self.get_call(destid)
+        call = self.get_call(dest)
         channels.dial(src, dest)
         call['stime'] = datetime.datetime.now()
         call['state'] = CALL_RINGING
@@ -299,10 +332,10 @@ class Calls:
         src_chan  = channels.get_channel(src)
         dest_chan = channels.get_channel(dest)
         cur.execute("INSERT INTO history (src, dest, stime, src_callerid, src_calleridname,"
-                    " dest_callerid, dest_calleridname, state, destid, srcid)"
-                    " VALUES (%s, %s, now(), %s, %s, %s, %s, %s, %s, %s)",
+                    " dest_callerid, dest_calleridname, state)"
+                    " VALUES (%s, %s, now(), %s, %s, %s, %s, %s)",
                     (src, dest, src_chan['callerid'], src_chan['calleridname'],
-                     dest_chan['callerid'], dest_chan['calleridname'], call['state'], destid, srcid))
+                     dest_chan['callerid'], dest_chan['calleridname'], call['state']))
         update = True
 
     def link(self, ev):
@@ -310,14 +343,22 @@ class Calls:
 
         src    = ev['Channel1']
         dest   = ev['Channel2']
-        destid = ev['Uniqueid2']
-        call = self.get_call(destid)
+        #destid = ev['Uniqueid2']
+        call = self.get_call(dest)
         channels.link(src, dest)
         call['ptime'] = datetime.datetime.now()
         call['state'] = CALL_TALKING
 
-        cur.execute("UPDATE history SET ptime = now(), state = %s WHERE destid = %s",
-                    (call['state'], destid))
+        cur.execute("UPDATE history SET ptime = now(), state = %s WHERE ptime IS NULL AND etime IS NULL AND src = %s AND dest = %s",
+                    (call['state'], src, dest))
+        if not cur.rowcount:
+            src_chan  = channels.get_channel(src)
+            dest_chan = channels.get_channel(dest)
+            cur.execute("INSERT INTO history (src, dest, stime, ptime, src_callerid, src_calleridname,"
+                        " dest_callerid, dest_calleridname, state)"
+                        " VALUES (%s, %s, now(), now(), %s, %s, %s, %s, %s)",
+                        (src, dest, src_chan['callerid'], src_chan['calleridname'],
+                         dest_chan['callerid'], dest_chan['calleridname'], call['state']))
         update = True
 
     def unlink(self, ev):
@@ -325,30 +366,33 @@ class Calls:
 
         src    = ev['Channel1']
         dest   = ev['Channel2']
-        destid = ev['Uniqueid2']
-        call = self.get_call(destid)
+        #destid = ev['Uniqueid2']
+        call = self.get_call(dest)
         call['etime'] = datetime.datetime.now()
         call['state'] = CALL_HANGUP
 
-        cur.execute("UPDATE history SET etime = now(), state = %s WHERE destid = %s",
-                    (call['state'], destid))
+        cur.execute("UPDATE history SET etime = now(), state = %s WHERE ptime IS NOT NULL and etime IS NULL AND dest = %s",
+                    (call['state'], dest))
         update = True
 
     def hangup(self, ev):
         global update
 
-        destid = ev['Uniqueid']
-        cause  = ev['Cause']
-        cur.execute("UPDATE history SET etime = now(), state = %s, cause = %s WHERE (srcid = %s OR destid = %s) AND etime IS NULL",
-                    (CALL_HANGUP, cause, destid, destid))
+        if ev['Channel'].find("<") != -1:                               # Bad hack, because we ignore channel renames
+            ev['Channel'] = ev['Channel'][:ev['Channel'].rfind("<")]    # Strip <MASQ> and <ZOMBIE>
+        #destid = ev['Uniqueid']
+        channel = ev['Channel']
+        cause   = ev['Cause']
+        cur.execute("UPDATE history SET etime = now(), state = %s, cause = %s WHERE (src = %s OR dest = %s) AND etime IS NULL",
+                    (CALL_HANGUP, cause, channel, channel))
         if cur.rowcount:
             update = True
 
-        if self.calls.has_key(destid):    # Try to hang up if channel is a valid destination
-            call = self.get_call(destid)
+        if self.calls.has_key(channel):    # Try to hang up if channel is a valid destination
+            call = self.get_call(channel)
             # call['etime'] = now()
             # call['state'] = CALL_HANGUP
-            del self.calls[destid]
+            del self.calls[channel]
 
         channels.hangup(ev)
 
@@ -356,14 +400,15 @@ class Calls:
         global update
 
         callerid, calleridname = channels.set_callerid(ev)
-        id = ev['Uniqueid']
+        #id = ev['Uniqueid']
+        channel = ev['Channel']
 
-        cur.execute("UPDATE history SET src_callerid = %s, src_calleridname = %s WHERE srcid = %s",
-                    (callerid, calleridname, id))
+        cur.execute("UPDATE history SET src_callerid = %s, src_calleridname = %s WHERE src = %s AND etime IS NULL",
+                    (callerid, calleridname, channel))
         if cur.rowcount:
             update = True
-        cur.execute("UPDATE history SET dest_callerid = %s, dest_calleridname = %s WHERE destid = %s",
-                    (callerid, calleridname, id))
+        cur.execute("UPDATE history SET dest_callerid = %s, dest_calleridname = %s WHERE dest = %s AND etime IS NULL",
+                    (callerid, calleridname, channel))
         if cur.rowcount:
             update = True
 
@@ -394,7 +439,7 @@ class AMI:
 
         # Get 'Asterisk Call Manager/1.0'
         ret = self.fd.readline()
-        if ret != 'Asterisk Call Manager/1.0\r\n':
+	if ret[:22] != 'Asterisk Call Manager/' or ret[25:] != '\r\n':
             raise Exception("Did not receive Asterisk Call Manager greeting.")
 
         ami_login = get_config_value('ami_login')
@@ -407,15 +452,43 @@ class AMI:
         if ev['Message'] != 'Authentication accepted':
             raise Exception("Asterisk Manager Interface - Authentication failed.")
 
+    def send_command(self, command):
+        query = 'Action: Command\r\nCommand: %s\r\n\r\n' % command
+        if config.debug: print query
+        self.fd.write(query)
+        self.fd.flush()
+
     def get_event(self):
         """
         Wait for an AMI event, put it in a dictionary and return it.
+        But first, send SIP Notifications for idle screen updates.
         """
+        #global gs_idle_updated, gs_idle_update
+        global gs_idle_update
+
+        if gs_idle_update:
+            peers = ''
+            for peer in gs_idle_update:
+		if peer[:4] == 'SIP/':
+                    peers += ' %s' % peer[4:]
+                    #gs_idle_updated[peer] = True
+            gs_idle_update = {}
+            if peers:
+                self.send_command('sip notify %s %s' % (config.idle_sipnotify, peers))
+
         ev = {}
-        line = self.fd.readline()[:-2]  # Strip trailing '\r\n'
+        line = self.fd.readline()
+        if line == '':
+                print "Possible disconnection."
+                raise socket.error()
+
+        line = line[:-2]  # Strip trailing '\r\n'
         while line != '':
-            key, value = line.split(': ', 1)
-            ev[key] = value
+            try:
+                key, value = line.split(': ', 1)
+                ev[key] = value
+            except ValueError:
+                if config.debug: print "Line not in key, value format: %s" % line
             line = self.fd.readline()[:-2]
         
         return ev
@@ -430,6 +503,10 @@ def mainloop(ami):
     cur.execute("COMMIT")
     while 1:
         ev = ami.get_event()
+        if not ev.has_key('Event'):
+            if config.debug: print ev
+            continue
+
         if ev['Event'] == 'Newexten':   # We don't need this.
             continue
 
